@@ -72,7 +72,7 @@ static PyObject *pg_default_passwd; /* default password */
 #endif  /* DEFAULT_VARS */
 
 static PyObject *decimal = NULL,    /* decimal type */
-                *dictiter = NULL,   /* function for getting named results */
+                *dictiter = NULL,   /* function for getting dict results */
                 *namediter = NULL,  /* function for getting named results */
                 *namednext = NULL,  /* function for getting one named result */
                 *scalariter = NULL, /* function for getting scalar results */
@@ -154,6 +154,7 @@ typedef struct
     PyObject_HEAD
     connObject *pgcnx;       /* parent connection object */
     PGresult   *result;      /* result content */
+    int        async;        /* flag for asynchronous queries */
     int        encoding;     /* client encoding */
     int        current_row;  /* currently selected row */
     int        max_row;      /* number of rows in the result */
@@ -197,7 +198,7 @@ typedef struct
 
 /* Connect to a database. */
 static char pg_connect__doc__[] =
-"connect(dbname, host, port, opt) -- connect to a PostgreSQL database\n\n"
+"connect(dbname, host, port, opt, user, passwd, wait) -- connect to a PostgreSQL database\n\n"
 "The connection uses the specified parameters (optional, keywords aware).\n";
 
 static PyObject *
@@ -205,16 +206,17 @@ pg_connect(PyObject *self, PyObject *args, PyObject *dict)
 {
     static const char *kwlist[] =
     {
-        "dbname", "host", "port", "opt", "user", "passwd", NULL
+        "dbname", "host", "port", "opt", "user", "passwd", "nowait", NULL
     };
 
     char *pghost, *pgopt, *pgdbname, *pguser, *pgpasswd;
-    int pgport;
+    int pgport = -1, nowait = 0, nkw = 0;
     char port_buffer[20];
+    const char *keywords[sizeof(kwlist) / sizeof(*kwlist) + 1],
+               *values[sizeof(kwlist) / sizeof(*kwlist) + 1];
     connObject *conn_obj;
 
     pghost = pgopt = pgdbname = pguser = pgpasswd = NULL;
-    pgport = -1;
 
     /*
      * parses standard arguments With the right compiler warnings, this
@@ -223,8 +225,8 @@ pg_connect(PyObject *self, PyObject *args, PyObject *dict)
      * I try to assign all those constant strings to it.
      */
     if (!PyArg_ParseTupleAndKeywords(
-        args, dict, "|zzizzz", (char**)kwlist,
-        &pgdbname, &pghost, &pgport, &pgopt, &pguser, &pgpasswd))
+        args, dict, "|zzizzzi", (char**)kwlist,
+        &pgdbname, &pghost, &pgport, &pgopt, &pguser, &pgpasswd, &nowait))
     {
         return NULL;
     }
@@ -250,7 +252,7 @@ pg_connect(PyObject *self, PyObject *args, PyObject *dict)
         pgpasswd = PyBytes_AsString(pg_default_passwd);
 #endif /* DEFAULT_VARS */
 
-    if (!(conn_obj = PyObject_NEW(connObject, &connType))) {
+    if (!(conn_obj = PyObject_New(connObject, &connType))) {
         set_error_msg(InternalError, "Can't create new connection object");
         return NULL;
     }
@@ -261,14 +263,44 @@ pg_connect(PyObject *self, PyObject *args, PyObject *dict)
     conn_obj->cast_hook = NULL;
     conn_obj->notice_receiver = NULL;
 
-    if (pgport != -1) {
+    if (pghost)
+    {
+        keywords[nkw] = "host";
+        values[nkw++] = pghost;
+    }
+    if (pgopt)
+    {
+        keywords[nkw] = "options";
+        values[nkw++] = pgopt;
+    }
+    if (pgdbname)
+    {
+        keywords[nkw] = "dbname";
+        values[nkw++] = pgdbname;
+    }
+    if (pguser)
+    {
+        keywords[nkw] = "user";
+        values[nkw++] = pguser;
+    }
+    if (pgpasswd)
+    {
+        keywords[nkw] = "password";
+        values[nkw++] = pgpasswd;
+    }
+    if (pgport != -1)
+    {
         memset(port_buffer, 0, sizeof(port_buffer));
         sprintf(port_buffer, "%d", pgport);
+
+        keywords[nkw] = "port";
+        values[nkw++] = port_buffer;
     }
+    keywords[nkw] = values[nkw] = NULL;
 
     Py_BEGIN_ALLOW_THREADS
-    conn_obj->cnx = PQsetdbLogin(pghost, pgport == -1 ? NULL : port_buffer,
-        pgopt, NULL, pgdbname, pguser, pgpasswd);
+    conn_obj->cnx = nowait ? PQconnectStartParams(keywords, values, 1) :
+        PQconnectdbParams(keywords, values, 1);
     Py_END_ALLOW_THREADS
 
     if (PQstatus(conn_obj->cnx) == CONNECTION_BAD) {
@@ -280,6 +312,19 @@ pg_connect(PyObject *self, PyObject *args, PyObject *dict)
     return (PyObject *) conn_obj;
 }
 
+#ifdef PQLIB_INFO
+
+/* Get version of libpq that is being used */
+static char pg_get_pqlib_version__doc__[] =
+"get_pqlib_version() -- get the version of libpq that is being used";
+
+static PyObject *
+pg_get_pqlib_version(PyObject *self, PyObject *noargs) {
+    return PyLong_FromLong(PQlibVersion());
+}
+
+#endif /* PQLIB_INFO */
+
 /* Escape string */
 static char pg_escape_string__doc__[] =
 "escape_string(string) -- escape a string for use within SQL";
@@ -310,20 +355,20 @@ pg_escape_string(PyObject *self, PyObject *string)
         return NULL;
     }
 
-    to_length = 2*from_length + 1;
+    to_length = 2 * (size_t) from_length + 1;
     if ((Py_ssize_t ) to_length < from_length) { /* overflow */
-        to_length = from_length;
+        to_length = (size_t) from_length;
         from_length = (from_length - 1)/2;
     }
     to = (char *) PyMem_Malloc(to_length);
-    to_length = (int) PQescapeString(to, from, (size_t) from_length);
+    to_length = (size_t) PQescapeString(to, from, (size_t) from_length);
 
     Py_XDECREF(tmp_obj);
 
     if (encoding == -1)
-        to_obj = PyBytes_FromStringAndSize(to, to_length);
+        to_obj = PyBytes_FromStringAndSize(to, (Py_ssize_t) to_length);
     else
-        to_obj = get_decoded_string(to, to_length, encoding);
+        to_obj = get_decoded_string(to, (Py_ssize_t) to_length, encoding);
     PyMem_Free(to);
     return to_obj;
 }
@@ -364,9 +409,9 @@ pg_escape_bytea(PyObject *self, PyObject *data)
     Py_XDECREF(tmp_obj);
 
     if (encoding == -1)
-        to_obj = PyBytes_FromStringAndSize(to, to_length - 1);
+        to_obj = PyBytes_FromStringAndSize(to, (Py_ssize_t) to_length - 1);
     else
-        to_obj = get_decoded_string(to, to_length - 1, encoding);
+        to_obj = get_decoded_string(to, (Py_ssize_t) to_length - 1, encoding);
     if (to)
         PQfreemem(to);
     return to_obj;
@@ -407,7 +452,7 @@ pg_unescape_bytea(PyObject *self, PyObject *data)
 
     if (!to) return PyErr_NoMemory();
 
-    to_obj = PyBytes_FromStringAndSize(to, to_length);
+    to_obj = PyBytes_FromStringAndSize(to, (Py_ssize_t) to_length);
     PQfreemem(to);
 
     return to_obj;
@@ -1176,7 +1221,6 @@ static struct PyMethodDef pg_methods[] = {
         METH_VARARGS|METH_KEYWORDS, pg_cast_record__doc__},
     {"cast_hstore", (PyCFunction) pg_cast_hstore,
         METH_O, pg_cast_hstore__doc__},
-
 #ifdef DEFAULT_VARS
     {"get_defhost", pg_get_defhost, METH_NOARGS, pg_get_defhost__doc__},
     {"set_defhost", pg_set_defhost, METH_VARARGS, pg_set_defhost__doc__},
@@ -1190,6 +1234,10 @@ static struct PyMethodDef pg_methods[] = {
     {"set_defuser", pg_set_defuser, METH_VARARGS, pg_set_defuser__doc__},
     {"set_defpasswd", pg_set_defpasswd, METH_VARARGS, pg_set_defpasswd__doc__},
 #endif /* DEFAULT_VARS */
+#ifdef PQLIB_INFO
+    {"get_pqlib_version", (PyCFunction) pg_get_pqlib_version,
+        METH_NOARGS, pg_get_pqlib_version__doc__},
+#endif /* PQLIB_INFO */
     {NULL, NULL} /* sentinel */
 };
 
@@ -1242,10 +1290,10 @@ MODULE_INIT_FUNC(_pg)
     dict = PyModule_GetDict(mod);
 
     /* Exceptions as defined by DB-API 2.0 */
-    Error = PyErr_NewException("pg.Error", PyExc_Exception, NULL);
+    Error = PyErr_NewException("pg.Error", PyExc_StandardError, NULL);
     PyDict_SetItemString(dict, "Error", Error);
 
-    Warning = PyErr_NewException("pg.Warning", PyExc_Exception, NULL);
+    Warning = PyErr_NewException("pg.Warning", PyExc_StandardError, NULL);
     PyDict_SetItemString(dict, "Warning", Warning);
 
     InterfaceError = PyErr_NewException(
@@ -1305,11 +1353,17 @@ MODULE_INIT_FUNC(_pg)
     PyDict_SetItemString(dict, "RESULT_DQL", PyInt_FromLong(RESULT_DQL));
 
     /* Transaction states */
-    PyDict_SetItemString(dict,"TRANS_IDLE",PyInt_FromLong(PQTRANS_IDLE));
-    PyDict_SetItemString(dict,"TRANS_ACTIVE",PyInt_FromLong(PQTRANS_ACTIVE));
-    PyDict_SetItemString(dict,"TRANS_INTRANS",PyInt_FromLong(PQTRANS_INTRANS));
-    PyDict_SetItemString(dict,"TRANS_INERROR",PyInt_FromLong(PQTRANS_INERROR));
-    PyDict_SetItemString(dict,"TRANS_UNKNOWN",PyInt_FromLong(PQTRANS_UNKNOWN));
+    PyDict_SetItemString(dict, "TRANS_IDLE", PyInt_FromLong(PQTRANS_IDLE));
+    PyDict_SetItemString(dict, "TRANS_ACTIVE", PyInt_FromLong(PQTRANS_ACTIVE));
+    PyDict_SetItemString(dict, "TRANS_INTRANS", PyInt_FromLong(PQTRANS_INTRANS));
+    PyDict_SetItemString(dict, "TRANS_INERROR", PyInt_FromLong(PQTRANS_INERROR));
+    PyDict_SetItemString(dict, "TRANS_UNKNOWN", PyInt_FromLong(PQTRANS_UNKNOWN));
+
+    /* Polling results */
+    PyDict_SetItemString(dict, "POLLING_OK", PyInt_FromLong(PGRES_POLLING_OK));
+    PyDict_SetItemString(dict, "POLLING_FAILED", PyInt_FromLong(PGRES_POLLING_FAILED));
+    PyDict_SetItemString(dict, "POLLING_READING", PyInt_FromLong(PGRES_POLLING_READING));
+    PyDict_SetItemString(dict, "POLLING_WRITING", PyInt_FromLong(PGRES_POLLING_WRITING));
 
 #ifdef LARGE_OBJECTS
     /* Create mode for large objects */
