@@ -272,7 +272,7 @@ source_oidstatus(sourceObject *self, PyObject *noargs)
         return Py_None;
     }
 
-    return PyInt_FromLong(oid);
+    return PyInt_FromLong((long) oid);
 }
 
 /* Fetch rows from last result. */
@@ -671,7 +671,7 @@ _source_buildinfo(sourceObject *self, int num)
     PyTuple_SET_ITEM(result, 1,
         PyStr_FromString(PQfname(self->result, num)));
     PyTuple_SET_ITEM(result, 2,
-        PyInt_FromLong(PQftype(self->result, num)));
+        PyInt_FromLong((long) PQftype(self->result, num)));
     PyTuple_SET_ITEM(result, 3,
         PyInt_FromLong(PQfsize(self->result, num)));
     PyTuple_SET_ITEM(result, 4,
@@ -711,7 +711,7 @@ source_listInfo(sourceObject *self, PyObject *noargs)
 
     /* returns result */
     return result;
-};
+}
 
 /* List fields information for last result. */
 static char source_fieldinfo__doc__[] =
@@ -732,7 +732,7 @@ source_fieldinfo(sourceObject *self, PyObject *desc)
 
     /* returns result */
     return _source_buildinfo(self, num);
-};
+}
 
 /* Retrieve field value. */
 static char source_field__doc__[] =
