@@ -1,4 +1,4 @@
-#! /usr/bin/python
+#!/usr/bin/python
 # -*- coding: utf-8 -*-
 
 """Test the classic PyGreSQL interface.
@@ -10,10 +10,7 @@ Contributed by Christoph Zwerschke.
 These tests do not need a database to test against.
 """
 
-try:
-    import unittest2 as unittest  # for Python < 2.7
-except ImportError:
-    import unittest
+import unittest
 
 import json
 import re
@@ -22,12 +19,12 @@ import pg  # the module under test
 
 from datetime import timedelta
 
-try:
+try:  # noinspection PyUnboundLocalVariable,PyUnresolvedReferences
     long
 except NameError:  # Python >= 3.0
     long = int
 
-try:
+try:  # noinspection PyUnboundLocalVariable,PyUnresolvedReferences
     unicode
 except NameError:  # Python >= 3.0
     unicode = str
@@ -66,6 +63,16 @@ class TestHasConnect(unittest.TestCase):
     def testhasPgNotSupportedError(self):
         self.assertTrue(issubclass(pg.NotSupportedError, pg.DatabaseError))
 
+    def testhasPgInvalidResultError(self):
+        self.assertTrue(issubclass(pg.InvalidResultError, pg.DataError))
+
+    def testhasPgNoResultError(self):
+        self.assertTrue(issubclass(pg.NoResultError, pg.InvalidResultError))
+
+    def testhasPgMultipleResultsError(self):
+        self.assertTrue(
+            issubclass(pg.MultipleResultsError, pg.InvalidResultError))
+
     def testhasConnect(self):
         self.assertTrue(callable(pg.connect))
 
@@ -114,6 +121,13 @@ class TestHasConnect(unittest.TestCase):
         pg.set_defbase(d0)
         self.assertEqual(pg.get_defbase(), d0)
 
+    def testPqlibVersion(self):
+        # noinspection PyUnresolvedReferences
+        v = pg.get_pqlib_version()
+        self.assertIsInstance(v, long)
+        self.assertGreater(v, 90000)
+        self.assertLess(v, 130000)
+
 
 class TestParseArray(unittest.TestCase):
     """Test the array parser."""
@@ -170,8 +184,8 @@ class TestParseArray(unittest.TestCase):
         (r"{abc,d,efg}", str, ['abc', 'd', 'efg']),
         ('{Hello World!}', str, ['Hello World!']),
         ('{Hello, World!}', str, ['Hello', 'World!']),
-        ('{Hello,\ World!}', str, ['Hello', ' World!']),
-        ('{Hello\, World!}', str, ['Hello, World!']),
+        (r'{Hello,\ World!}', str, ['Hello', ' World!']),
+        (r'{Hello\, World!}', str, ['Hello, World!']),
         ('{"Hello World!"}', str, ['Hello World!']),
         ('{this, should, be, null}', str, ['this', 'should', 'be', None]),
         ('{This, should, be, NULL}', str, ['This', 'should', 'be', None]),
@@ -192,11 +206,11 @@ class TestParseArray(unittest.TestCase):
         ('{{{17,18,19},{14,15,16},{11,12,13}},'
          '{{27,28,29},{24,25,26},{21,22,23}},'
          '{{37,38,39},{34,35,36},{31,32,33}}}', int,
-            [[[17, 18, 19], [14, 15, 16], [11, 12, 13]],
-             [[27, 28, 29], [24, 25, 26], [21, 22, 23]],
-             [[37, 38, 39], [34, 35, 36], [31, 32, 33]]]),
+         [[[17, 18, 19], [14, 15, 16], [11, 12, 13]],
+          [[27, 28, 29], [24, 25, 26], [21, 22, 23]],
+          [[37, 38, 39], [34, 35, 36], [31, 32, 33]]]),
         ('{{"breakfast", "consulting"}, {"meeting", "lunch"}}', str,
-            [['breakfast', 'consulting'], ['meeting', 'lunch']]),
+         [['breakfast', 'consulting'], ['meeting', 'lunch']]),
         ('[1:3]={1,2,3}', int, [1, 2, 3]),
         ('[-1:1]={1,2,3}', int, [1, 2, 3]),
         ('[-1:+1]={1,2,3}', int, [1, 2, 3]),
@@ -208,9 +222,9 @@ class TestParseArray(unittest.TestCase):
         ('[1:]={1,2,3}', int, ValueError),
         ('[:3]={1,2,3}', int, ValueError),
         ('[1:1][-2:-1][3:5]={{{1,2,3},{4,5,6}}}',
-            int, [[[1, 2, 3], [4, 5, 6]]]),
+         int, [[[1, 2, 3], [4, 5, 6]]]),
         ('  [1:1]  [-2:-1]  [3:5]  =  { { { 1 , 2 , 3 }, {4 , 5 , 6 } } }',
-            int, [[[1, 2, 3], [4, 5, 6]]]),
+         int, [[[1, 2, 3], [4, 5, 6]]]),
         ('[1:1][3:5]={{1,2,3},{4,5,6}}', int, [[1, 2, 3], [4, 5, 6]]),
         ('[3:5]={{1,2,3},{4,5,6}}', int, ValueError),
         ('[1:1][-2:-1][3:5]={{1,2,3},{4,5,6}}', int, ValueError)]
@@ -268,6 +282,7 @@ class TestParseArray(unittest.TestCase):
         for i in range(7):
             self.assertIsInstance(r, list)
             self.assertEqual(len(r), 1)
+            # noinspection PyUnresolvedReferences
             r = r[0]
         self.assertEqual(r, 'abc')
 
@@ -296,7 +311,9 @@ class TestParseArray(unittest.TestCase):
         self.assertEqual(f('{a}', None), ['a'])
         self.assertRaises(ValueError, f, '{a}', int)
         self.assertEqual(f('{a}', str), ['a'])
-        cast = lambda s: '%s is ok' % s
+
+        def cast(s):
+            return '%s is ok' % s
         self.assertEqual(f('{a}', cast), ['a is ok'])
 
     def testParserDelim(self):
@@ -439,8 +456,8 @@ class TestParseRecord(unittest.TestCase):
         (r"(\'abc\')", str, ("'abc'",)),
         ('(Hello World!)', str, ('Hello World!',)),
         ('(Hello, World!)', str, ('Hello', ' World!',)),
-        ('(Hello,\ World!)', str, ('Hello', ' World!',)),
-        ('(Hello\, World!)', str, ('Hello, World!',)),
+        (r'(Hello,\ World!)', str, ('Hello', ' World!',)),
+        (r'(Hello\, World!)', str, ('Hello, World!',)),
         ('("Hello World!")', str, ('Hello World!',)),
         ("(this,shouldn't,be,null)", str, ('this', "shouldn't", 'be', 'null')),
         ('(null,should,be,)', str, ('null', 'should', 'be', None)),
@@ -536,7 +553,9 @@ class TestParseRecord(unittest.TestCase):
         self.assertEqual(f('(a)', None), ('a',))
         self.assertRaises(ValueError, f, '(a)', int)
         self.assertEqual(f('(a)', str), ('a',))
-        cast = lambda s: '%s is ok' % s
+
+        def cast(s):
+            return '%s is ok' % s
         self.assertEqual(f('(a)', cast), ('a is ok',))
 
     def testParserCastNonUniform(self):
@@ -555,19 +574,25 @@ class TestParseRecord(unittest.TestCase):
         self.assertRaises(ValueError, f, '(1,a)', [str, int])
         self.assertEqual(f('(a,1)', [str, int]), ('a', 1))
         self.assertRaises(ValueError, f, '(a,1)', [int, str])
-        self.assertEqual(f('(1,a,2,b,3,c)',
-            [int, str, int, str, int, str]), (1, 'a', 2, 'b', 3, 'c'))
-        self.assertEqual(f('(1,a,2,b,3,c)',
-            (int, str, int, str, int, str)), (1, 'a', 2, 'b', 3, 'c'))
-        cast1 = lambda s: '%s is ok' % s
+        self.assertEqual(
+            f('(1,a,2,b,3,c)', [int, str, int, str, int, str]),
+            (1, 'a', 2, 'b', 3, 'c'))
+        self.assertEqual(
+            f('(1,a,2,b,3,c)', (int, str, int, str, int, str)),
+            (1, 'a', 2, 'b', 3, 'c'))
+
+        def cast1(s):
+            return '%s is ok' % s
         self.assertEqual(f('(a)', [cast1]), ('a is ok',))
-        cast2 = lambda s: 'and %s is ok, too' % s
-        self.assertEqual(f('(a,b)', [cast1, cast2]),
-            ('a is ok', 'and b is ok, too'))
+
+        def cast2(s):
+            return 'and %s is ok, too' % s
+        self.assertEqual(
+            f('(a,b)', [cast1, cast2]), ('a is ok', 'and b is ok, too'))
         self.assertRaises(ValueError, f, '(a)', [cast1, cast2])
         self.assertRaises(ValueError, f, '(a,b,c)', [cast1, cast2])
-        self.assertEqual(f('(1,2,3,4,5,6)',
-            [int, float, str, None, cast1, cast2]),
+        self.assertEqual(
+            f('(1,2,3,4,5,6)', [int, float, str, None, cast1, cast2]),
             (1, 2.0, '3', '4', '5 is ok', 'and 6 is ok, too'))
 
     def testParserDelim(self):
@@ -643,14 +668,13 @@ class TestParseHStore(unittest.TestCase):
         ('"k=>v', ValueError),
         ('k=>"v', ValueError),
         ('"1-a" => "anything at all"', {'1-a': 'anything at all'}),
-        ('k => v, foo => bar, baz => whatever,'
-                ' "1-a" => "anything at all"',
-            {'k': 'v', 'foo': 'bar', 'baz': 'whatever',
-            '1-a': 'anything at all'}),
+        ('k => v, foo => bar, baz => whatever, "1-a" => "anything at all"',
+         {'k': 'v', 'foo': 'bar', 'baz': 'whatever',
+          '1-a': 'anything at all'}),
         ('"Hello, World!"=>"Hi!"', {'Hello, World!': 'Hi!'}),
         ('"Hi!"=>"Hello, World!"', {'Hi!': 'Hello, World!'}),
-        ('"k=>v"=>k\=\>v', {'k=>v': 'k=>v'}),
-        ('k\=\>v=>"k=>v"', {'k=>v': 'k=>v'}),
+        (r'"k=>v"=>k\=\>v', {'k=>v': 'k=>v'}),
+        (r'k\=\>v=>"k=>v"', {'k=>v': 'k=>v'}),
         ('a\\,b=>a,b=>a', {'a,b': 'a', 'b': 'a'})]
 
     def testParser(self):
@@ -678,10 +702,10 @@ class TestCastInterval(unittest.TestCase):
             ('-1:00:00', '-01:00:00', '@ -1 hour', 'PT-1H')),
         ((0, 0, 0, 1, 0, 0, 0),
             ('0-0 0 1:00:00', '0 years 0 mons 0 days 01:00:00',
-            '@ 0 years 0 mons 0 days 1 hour', 'P0Y0M0DT1H')),
+             '@ 0 years 0 mons 0 days 1 hour', 'P0Y0M0DT1H')),
         ((0, 0, 0, -1, 0, 0, 0),
             ('-0-0 -1:00:00', '0 years 0 mons 0 days -01:00:00',
-            '@ 0 years 0 mons 0 days -1 hour', 'P0Y0M0DT-1H')),
+             '@ 0 years 0 mons 0 days -1 hour', 'P0Y0M0DT-1H')),
         ((0, 0, 1, 0, 0, 0, 0),
             ('1 0:00:00', '1 day', '@ 1 day', 'P1D')),
         ((0, 0, -1, 0, 0, 0, 0),
@@ -835,7 +859,8 @@ class TestCastInterval(unittest.TestCase):
             f = pg.cast_interval
             years, mons, days, hours, mins, secs, usecs = result
             days += 365 * years + 30 * mons
-            interval = timedelta(days=days, hours=hours, minutes=mins,
+            interval = timedelta(
+                days=days, hours=hours, minutes=mins,
                 seconds=secs, microseconds=usecs)
             for value in values:
                 self.assertEqual(f(value), interval)
@@ -906,7 +931,7 @@ class TestConfigFunctions(unittest.TestCase):
     def testGetDatestyle(self):
         self.assertIsNone(pg.get_datestyle())
 
-    def testGetDatestyle(self):
+    def testSetDatestyle(self):
         datestyle = pg.get_datestyle()
         try:
             pg.set_datestyle('ISO, YMD')
@@ -1005,27 +1030,6 @@ class TestConfigFunctions(unittest.TestCase):
         self.assertIsInstance(r, bool)
         self.assertIs(r, bytea_escaped)
 
-    def testGetNamedresult(self):
-        r = pg.get_namedresult()
-        self.assertTrue(callable(r))
-        self.assertIs(r, pg._namedresult)
-
-    def testSetNamedresult(self):
-        namedresult = pg.get_namedresult()
-        try:
-            pg.set_namedresult(None)
-            r = pg.get_namedresult()
-            self.assertIsNone(r)
-            f = lambda q: q.getresult()
-            pg.set_namedresult(f)
-            r = pg.get_namedresult()
-            self.assertIs(r, f)
-            self.assertRaises(TypeError, pg.set_namedresult, 'invalid')
-        finally:
-            pg.set_namedresult(namedresult)
-        r = pg.get_namedresult()
-        self.assertIs(r, namedresult)
-
     def testGetJsondecode(self):
         r = pg.get_jsondecode()
         self.assertTrue(callable(r))
