@@ -1,4 +1,4 @@
-#! /usr/bin/python
+#!/usr/bin/python
 # -*- coding: utf-8 -*-
 
 """Test the modern PyGreSQL interface.
@@ -10,12 +10,9 @@ Contributed by Christoph Zwerschke.
 These tests need a database to test against.
 """
 
-try:
-    import unittest2 as unittest  # for Python < 2.7
-except ImportError:
-    import unittest
+import unittest
 
-try:
+try:   # noinspection PyCompatibility
     from collections.abc import Iterable
 except ImportError:  # Python < 3.3
     from collections import Iterable
@@ -30,14 +27,14 @@ dbhost = None
 dbport = 5432
 
 try:
-    from .LOCAL_PyGreSQL import *
+    from .LOCAL_PyGreSQL import *  # noqa: F401
 except (ImportError, ValueError):
     try:
-        from LOCAL_PyGreSQL import *
+        from LOCAL_PyGreSQL import *  # noqa: F401
     except ImportError:
         pass
 
-try:
+try:  # noinspection PyUnboundLocalVariable,PyUnresolvedReferences
     unicode
 except NameError:  # Python >= 3.0
     unicode = str
@@ -129,8 +126,8 @@ class TestCopy(unittest.TestCase):
 
     @staticmethod
     def connect():
-        return pgdb.connect(database=dbname,
-            host='%s:%d' % (dbhost or '', dbport or -1))
+        return pgdb.connect(
+            database=dbname, host='%s:%d' % (dbhost or '', dbport or -1))
 
     @classmethod
     def setUpClass(cls):
@@ -139,7 +136,7 @@ class TestCopy(unittest.TestCase):
         cur.execute("set client_min_messages=warning")
         cur.execute("drop table if exists copytest cascade")
         cur.execute("create table copytest ("
-            "id smallint primary key, name varchar(64))")
+                    "id smallint primary key, name varchar(64))")
         cur.close()
         con.commit()
         cur = con.cursor()
@@ -247,8 +244,8 @@ class TestCopyFrom(TestCopy):
         self.assertRaises(TypeError, call, '0\t', 'copytest', null=42)
         self.assertRaises(ValueError, call, '0\t', 'copytest', size='bad')
         self.assertRaises(TypeError, call, '0\t', 'copytest', columns=42)
-        self.assertRaises(ValueError, call, b'', 'copytest',
-            format='binary', sep=',')
+        self.assertRaises(
+            ValueError, call, b'', 'copytest', format='binary', sep=',')
 
     def test_input_string(self):
         ret = self.copy_from('42\tHello, world!')
@@ -284,6 +281,7 @@ class TestCopyFrom(TestCopy):
             self.copy_from(u'43\tWürstel, Käse!')
             self.assertEqual(self.table_data, [(43, 'Würstel, Käse!')])
             self.truncate_table()
+            # noinspection PyUnresolvedReferences
             self.copy_from(self.data_text.decode('utf-8'))
             self.check_table()
 
@@ -303,7 +301,7 @@ class TestCopyFrom(TestCopy):
 
         def test_input_iterable_bytes(self):
             self.copy_from(row.encode('utf-8')
-                for row in self.data_text.splitlines())
+                           for row in self.data_text.splitlines())
             self.check_table()
 
     def test_sep(self):
@@ -342,7 +340,7 @@ class TestCopyFrom(TestCopy):
             (1, None), (2, None), (3, 'Three'), (4, 'Four'), (5, 'Five')])
         self.check_rowcount(5)
         self.assertRaises(pgdb.ProgrammingError, self.copy_from,
-            '6\t42', columns=['id', 'age'])
+                          '6\t42', columns=['id', 'age'])
         self.check_rowcount(-1)
 
     def test_csv(self):
@@ -356,13 +354,13 @@ class TestCopyFrom(TestCopy):
         self.check_rowcount()
 
     def test_binary(self):
-        self.assertRaises(IOError, self.copy_from,
-            b'NOPGCOPY\n', format='binary')
+        self.assertRaises(
+            IOError, self.copy_from, b'NOPGCOPY\n', format='binary')
         self.check_rowcount(-1)
 
     def test_binary_with_sep(self):
-        self.assertRaises(ValueError, self.copy_from,
-            '', format='binary', sep='\t')
+        self.assertRaises(
+            ValueError, self.copy_from, '', format='binary', sep='\t')
 
     def test_binary_with_unicode(self):
         self.assertRaises(ValueError, self.copy_from, u'', format='binary')
@@ -398,8 +396,8 @@ class TestCopyFrom(TestCopy):
         self.check_rowcount()
 
     def test_size_invalid(self):
-        self.assertRaises(TypeError,
-            self.copy_from, self.data_file, size='invalid')
+        self.assertRaises(
+            TypeError, self.copy_from, self.data_file, size='invalid')
 
 
 class TestCopyTo(TestCopy):
@@ -471,6 +469,7 @@ class TestCopyTo(TestCopy):
             self.assertEqual(len(rows), 3)
             rows = ''.join(rows)
             self.assertIsInstance(rows, unicode)
+            # noinspection PyUnresolvedReferences
             self.assertEqual(rows, self.data_text.decode('utf-8'))
 
     def test_rowcount_increment(self):
@@ -519,8 +518,8 @@ class TestCopyTo(TestCopy):
         self.assertEqual(ret, self.data_text)
         ret = ''.join(self.copy_to(columns=['id', 'name']))
         self.assertEqual(ret, self.data_text)
-        self.assertRaises(pgdb.ProgrammingError, self.copy_to,
-            columns=['id', 'age'])
+        self.assertRaises(
+            pgdb.ProgrammingError, self.copy_to, columns=['id', 'age'])
 
     def test_csv(self):
         ret = self.copy_to(format='csv')
@@ -548,14 +547,15 @@ class TestCopyTo(TestCopy):
         self.assertRaises(ValueError, self.copy_to, format='binary', sep='\t')
 
     def test_binary_with_unicode(self):
-        self.assertRaises(ValueError, self.copy_to,
-            format='binary', decode=True)
+        self.assertRaises(
+            ValueError, self.copy_to, format='binary', decode=True)
 
     def test_query(self):
-        self.assertRaises(ValueError, self.cursor.copy_to, None,
+        self.assertRaises(
+            ValueError, self.cursor.copy_to, None,
             "select name from copytest", columns='noname')
-        ret = self.cursor.copy_to(None,
-            "select name||'!' from copytest where id=1941")
+        ret = self.cursor.copy_to(
+            None, "select name||'!' from copytest where id=1941")
         self.assertIsInstance(ret, Iterable)
         rows = list(ret)
         self.assertEqual(len(rows), 1)
